'''
Author:        Boning Dong
Last Modified: 7/7/2015

This script can allow users to add randomly generate bricks to
a water tank in PAGI world repeatedly. After adding bricks, the
coordinate of the bricks will be observed by PAGI guy's visual
field, and will be saved to a text file with all other data. Data
for each iteration holds am individual line in the output file. 
The format of each line is shown below.

mass,width,height,state,r,g,b,[[coordinates],[coordinates]...]\n

#for state, 1 is sinking, 3 is floating and 2 is bearly moving 
#r,g,b is a randomly generate rgb color 

*in order to use this script, PIL module for python is required
'''

import math
import random
import socket
import time
from PIL import Image


def send(txt, num = 256):
    # send data through socket
    clientsocket.send(txt+'\n')
    # receive feedback
    ans = clientsocket.recv(num).split(',')
    return ans

def prepare():
    # create thin brick above PAGI
    send('createItem,thinwall,brickWallHorizontal.png,-15.7,-1.6,5,1,0,0,1')
    # add water
    send('dropItem,fluid,-11,10')
    send('dropItem,fluid,-11,10')
    
def resizeImage(x, y):
    # create a temporay image with the given height and width
    # ratio

    # open the brick image
    img = Image.open('brickWall.png')
    # calculate the size for new image
    xx = int(x * float(img.size[0]))
    yy = int(y * float(img.size[1]))
    # resize image
    img = img.resize((xx, yy), Image.ANTIALIAS)
    # save image
    img.save('obj_temp.png')   

def find(code, mode, num = 128):
    # find the object in the visual field
    
    if not isinstance(num, int):
        num = 128        
    
    ans = send('findObj,'+code+','+mode)
    ans[-1] = ans[-1].strip()
    return ans

def create(name, x, y, mass):
    # create item in PAGI world
    send('createItem,'+name+',obj_temp.png,'+str(x)+','+str(y)+','+str(mass)+',1,0,0,5') 

def addToWater1(x, y, num):
    # add a brick with random mass and random size to the
    # water, judge whether it will sink or float
    # x and y is the coordinate for adding the brick, and
    # num is the time of iteration

    # get global variable
    global text_file
    for i in range(num):
        # randomly genreate the mass
        mass = random.uniform(1, 3)
        mass = math.pow(10, mass)
        
        # randomly generate the size
        xx = random.uniform(0.1, 1)
        yy = random.uniform(0.1, 1)
        
        # create the image with the generated size
        resizeImage(xx, yy)

        # add the brick to the water
        create('obj1', x, y, mass)
    
        #  save data
        ini = send('findObj,obj1,D', 4096)
        temp = []
        temp2 = []
        t1 = time.time();
        for j in range(100):
            # collect data through socket
            temp.append(find('obj1', 'P'))
        fin = send('findObj,obj1,D', 4096)
        print time.time() - t1,

        # format data
        temp = [i[2:] for i in temp]

        for j in temp:
            if j != []:
                temp2.append(j)

        stat = 0

        # check whether it sinks or float according to
        # initial and final condition
        # calculate the average height of the object in 
        # detailed visual field for initial condition
        ini_avg = 0
        n = 0
        for j in ini[2:]:
            ini_avg += int(j.split('.')[-1])
            n += 1
        if n != 0:
            ini_avg = ini_avg/float(n)

        # calculate the average height of the object in 
        # detailed visual field for final condition
        fin_avg = 0
        n = 0
        for j in fin[2:]:
            fin_avg += int(j.split('.')[-1])
            n += 1
        if n != 0:
            fin_avg = fin_avg/float(n)

        # compare the average height
        if ini_avg != 0:
            if ini_avg > fin_avg:
                # if sink
                if fin_avg != 0:
                    stat = 1
                # if exit the detailed visual field
                else:
                    stat = 3
            elif ini_avg == fin_avg:
                # if bearly move
                stat = 2
            else:
                # if float
                stat = 3

            # randomly generate the RGB color for the brick
            r = random.randint(0, 255)
            g = random.randint(0, 255)
            b = random.randint(0, 255)

            # save data to string
            dstr = '%s,%s,%s,%s,%s,%s,%s' %(mass,xx,yy,stat,r , g, b)
            dstr += str(temp2) + '\n'

            print stat, ini_avg, fin_avg

            # write the string to file
            text_file.write(dstr)

            # destroy the item
            send('destroyItem,obj1')
            time.sleep(1)

if __name__ == '__main__':    
    #  connect to local address
    IP_ADDRESS = socket.gethostbyname(socket.gethostname())
    # IP_ADDRESS = '129.161.58.120'
    clientsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    clientsocket.connect((IP_ADDRESS, 42209))

    random.seed()

    # prepare water
    prepare()
    time.sleep(5)
    
    # open file
    text_file = open('floating_output.txt','a')
    # start
    addToWater1(-12, 0, 200000)
    # close file
    text_file.close()